import numpy as np
import cv2 as cv
import matplotlib.pyplot as plt

PI = 3.141592653589793238

class Colors:
    RED = "RED"
    BLUE = "BLUE"
    GREY = "GREY"

class GreyBlobDetector:
    # we know that if an armor is either blue or red, it must be white except on the edges
    # since the shape does not change, we can try to find white blobs with the desired features

    # threshold to be considered white; 0 = black 255 = white, TBD
    WHITE_THRESHOLD = 235
    # threshold for circularity (circle = 1); TBD
    # circularity = 4 * PI * AREA / (PERIMETER ^ 2)
    CIRCLE_THRESHOLD = 0.5
    # threshold for concavity; the formula is area_of_blob / area_convexity_hull, TBD
    CONVEX_THRESHOLD = 0.99
    # threshold for how elongated it is (called intertia in opencv), TBD (circle = 1)
    INERTIA_THRESHOLD = 0.2
    # unless a robot got flipped, the lines should be pretty vertical
    ANGLE_THRESHOLD = 30
    # these parameters are for the shape of the image, not sure if needed or not
    WIDTH_RATIO = 0.2 # width LED / width image, TBD
    HEIGHT_RATIO = 0.67 # height LED / height image, TBD
    DIST_LEDS_RATIO = 0.67 # distance of the 2 LEDs / width of image, TBD



    def __init__(self):
        pass

    def calculate_max_area(self, image):
        # this basically calcultes the min / max area 
        # false by default unless the ai that finds the armors is consistant enough with the
        # picture horizontal and vertical ratio
        x, y = image.shape
        return (x * GreyBlobDetector.WIDTH_RATIO)*(y * GreyBlobDetector.HEIGHT_RATIO) 

    def calculate_min_area(self, image):
        # needed here becase part of the vertical bars may be hidden
        return self.calculate_max_area(image) / 1.5 # TBD

    def calculate_distance(self, image):
        # only use this if there are 2 or more blobs detected
        x, y = image.shape
        return y * GreyBlobDetector.DIST_LEDS_RATIO # TBD

    # finds the contours of the LEDs and returns them
    def detect_color(self, image, area = False, distance = False, already_grayscale = False):
        # transform the image into black and white
        # first go to grayscale then goto black white with a threshold
        grey = image
        if not already_grayscale:
            grey = cv.cvtColor(image, cv.COLOR_BGR2GRAY)
        black_white = cv.threshold(grey, GreyBlobDetector.WHITE_THRESHOLD, 255, cv.THRESH_BINARY)[1]

        # do find countours on the black white image
        contours, hierarchy = cv.findContours(black_white, cv.RETR_TREE, cv.CHAIN_APPROX_SIMPLE)
        # note : other versions of OpenCV will require image, contours, hierarchy
        # now for the characteristics of the contours
        retained_contours = []
        for contour in contours:
            area = cv.contourArea(contour)
            # circularity < threshold : is the contour NOT circular?
            circularity = 4*PI*area / (len(contour) * len(contour))
            # convexity > threashold : we want the shape to be convex
            convexity = area / cv.contourArea(cv.convexHull(contour))
            # opencv docs aren't very helpful on this but looks like inertia is aspect ratio
            (x,y),(width,height),theta = cv.minAreaRect(contour)
            # this is basically how e
            inertia = width / height
            
            if circularity < GreyBlobDetector.CIRCLE_THRESHOLD and \
                convexity > GreyBlobDetector.CONVEX_THRESHOLD and \
                inertia < GreyBlobDetector.INERTIA_THRESHOLD and \
                theta < GreyBlobDetector.ANGLE_THRESHOLD:
                retained_contours.append(contour)
        
        return retained_contours
            
if __name__ == '__main__':
    image = cv.imread("test.png")
    